//+------------------------------------------------------------------+
//|                                                     MACD_CCI.mq4 |
//+------------------------------------------------------------------+

#property  indicator_separate_window
#property  indicator_buffers 3
#property  indicator_color1  RoyalBlue
#property  indicator_color2  Red
#property  indicator_color3  Gold
#property  indicator_minimum 0
#property  indicator_maximum 77
#property  indicator_width1  2
#property  indicator_width2  2
#property  indicator_width3  2

//
//
//
//
//

extern string TimeFrame  = "current time frame";
extern int    macdMAfast = 144;
extern int    macdMAslow = 233;
extern int    macdSigMA  =  21;
extern int    macdPrice  =   0;
extern int    macdLnMode =   0;
extern int    CCIperiod  =  50;
extern int    CCIprice   =   5;
extern double BarLevl    =   5;

//
//
//
//
//

extern string MA_Method_Price = "SMA0 EMA1 SMMA2 LWMA3||0O,1C 2H3L,4Md 5Tp 6WghC: Md(HL/2)4,Tp(HLC/3)5,Wgh(HLCC/4)6||macdLnMode:0main 1sig";

extern bool   alertsOn          = true;
extern bool   alertsOnCurrent   = true;
extern bool   alertsMessage     = true;
extern bool   alertsSound       = false;
extern bool   alertsEmail       = false;

//
//
//
//
//

double buffer1[];
double buffer2[];
double buffer3[];

int      timeFrame;
datetime lastABar;
bool     returningBars=false;
bool     calculatingMacd=false;
string   IndicatorFileName;
string   timeFrameDescription="current time frame";

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   SetIndexBuffer(0,buffer1); SetIndexStyle(0,DRAW_ARROW); SetIndexArrow(0,167); SetIndexEmptyValue(0,EMPTY_VALUE); 
   SetIndexBuffer(1,buffer2); SetIndexStyle(1,DRAW_ARROW); SetIndexArrow(1,167); SetIndexEmptyValue(1,EMPTY_VALUE);
   SetIndexBuffer(2,buffer3); SetIndexStyle(2,DRAW_ARROW); SetIndexArrow(2,167); SetIndexEmptyValue(2,EMPTY_VALUE); 

      if (TimeFrame=="calculateMACD") { calculatingMacd=true; return(0); }
      if (TimeFrame=="returnBars")    { returningBars=true;   return(0); }
      
   //
   //
   //
   //
   //
      
      timeFrame            = stringToTimeFrame(TimeFrame);
      timeFrameDescription = timeFrameToString(timeFrame);
      IndicatorFileName    = WindowExpertName();

      string name1= " macd ("+macdMAfast+","+macdMAslow+","+macdSigMA+")";
      string name2= " cci (" +CCIperiod+")";
   IndicatorShortName("Zerolag MACD-CCI "+timeFrameToString(timeFrame)+name1+name2);  
   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int start()
{
   int i,limit,counted_bars = IndicatorCounted();

   if(counted_bars < 0) return(-1);
   if(counted_bars > 0) counted_bars--;
      limit = MathMin(Bars - counted_bars,Bars-1);

      if (returningBars)   { buffer1[0] = limit+1; return(0); }
      if (calculatingMacd) { CalculateMacd(limit,macdLnMode); return(0); }

   //
   //
   //
   //
   //
      
   if (timeFrame > Period()) limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,IndicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   for (i=limit; i>= 0; i--)
   {
      int y =iBarShift(NULL,timeFrame,Time[i]);
         double  macd0 = iCustom(NULL,timeFrame,IndicatorFileName,"calculateMACD",macdMAfast,macdMAslow,macdSigMA,macdPrice,macdLnMode,0,y);
         double  macd1 = iCustom(NULL,timeFrame,IndicatorFileName,"calculateMACD",macdMAfast,macdMAslow,macdSigMA,macdPrice,macdLnMode,0,y+1);
         double  cci   = iCCI (NULL,timeFrame,CCIperiod,CCIprice,y);

         buffer1[i]=EMPTY_VALUE;
         buffer2[i]=EMPTY_VALUE;
         buffer3[i]=EMPTY_VALUE;
      
         if (cci > 0 && macd0 > macd1) buffer1[i] = BarLevl;
         else 
            if (cci < 0 && macd0 < macd1) buffer2[i] = BarLevl;  
            else                          buffer3[i] = BarLevl;
   }
   
   //
   //
   //
   //
   //

   if (alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1; whichBar = iBarShift(NULL,0,iTime(NULL,timeFrame,whichBar));

      //
      //
      //
      //
      //
         
      if (buffer1[whichBar] != EMPTY_VALUE && buffer1[whichBar+1] == EMPTY_VALUE)
            doAlert(whichBar,timeFrameDescription+" change color to blue");
      if (buffer2[whichBar] != EMPTY_VALUE && buffer2[whichBar+1] == EMPTY_VALUE)
            doAlert(whichBar,timeFrameDescription+" change color to red");
   }
   return(0);
}


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

double workBuffer[][6];
#define ema11 0
#define ema12 1
#define ema21 2
#define ema22 3
#define ema31 4
#define ema32 5

//
//
//
//
//

void CalculateMacd(int limit, int mode)
{
   double alpha1 = 2.0/(1.0+macdMAfast);
   double alpha2 = 2.0/(1.0+macdMAslow);
   double alpha3 = 2.0/(1.0+macdSigMA);
   int i,r;

      if (ArrayRange(workBuffer,0)!=Bars) ArrayResize(workBuffer,Bars);
      
   //
   //
   //
   //
   //
         
   for (i=limit, r=Bars-i-1; i>= 0; i--,r++)
   {
      double price = iMA(NULL,0,1,0,MODE_SMA,macdPrice,i);
      if (i==(Bars-1))
      {
         workBuffer[r][ema11] = price;
         workBuffer[r][ema12] = price;
         workBuffer[r][ema21] = price;
         workBuffer[r][ema22] = price;
         continue;
      }
      workBuffer[r][ema11] = workBuffer[r-1][ema11]+alpha1*(price               -workBuffer[r-1][ema11]);
      workBuffer[r][ema12] = workBuffer[r-1][ema12]+alpha1*(workBuffer[r][ema11]-workBuffer[r-1][ema12]);
      workBuffer[r][ema21] = workBuffer[r-1][ema21]+alpha2*(price               -workBuffer[r-1][ema21]);
      workBuffer[r][ema22] = workBuffer[r-1][ema22]+alpha2*(workBuffer[r][ema21]-workBuffer[r-1][ema22]);
      double macd          = (2.0*workBuffer[r][ema11]-workBuffer[r][ema12])-
                             (2.0*workBuffer[r][ema21]-workBuffer[r][ema22]);
                             
      //
      //
      //
      //
      //

      if (mode==1)
      {      
         workBuffer[r][ema31] = workBuffer[r-1][ema31]+alpha3*(macd                -workBuffer[r-1][ema31]);
         workBuffer[r][ema32] = workBuffer[r-1][ema32]+alpha3*(workBuffer[r][ema31]-workBuffer[r-1][ema32]);
         buffer1[i]           = (2.0*workBuffer[r][ema31]-workBuffer[r][ema32]);
      }
      else buffer1[i] = macd;         
   }
}


//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[forBar]) {
       previousAlert  = doWhat;
       previousTime   = Time[forBar];

       //
       //
       //
       //
       //

       message =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," Indicator MACD_CCI ",doWhat);
          if (alertsMessage) Alert(message);
          if (alertsEmail)   SendMail(StringConcatenate(Symbol(),"Indicator MACD_CCI "),message);
          if (alertsSound)   PlaySound("alert2.wav");
   }
}

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = StringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string StringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int tchar = StringGetChar(s, length);
         if((tchar > 96 && tchar < 123) || (tchar > 223 && tchar < 256))
                     s = StringSetChar(s, length, tchar - 32);
         else if(tchar > -33 && tchar < 0)
                     s = StringSetChar(s, length, tchar + 224);
   }
   return(s);
}